import os
import unittest
from dataclasses import replace
from io import StringIO
from pathlib import Path
from unittest.mock import patch

from colors import red, yellow
from packaging import version
from packaging.version import Version

from daktari import __version__
from daktari.version_utils import sanitise_version_string
from daktari.checks.intellij_idea import IntelliJIdeaInstalled, IntelliJProjectImported
from daktari.checks.misc import EnvVarSet
from daktari.config import (
    check_version_compatibility,
    parse_raw_config,
    LOCAL_CONFIG_PATH,
    Config,
    apply_local_config,
    write_local_config_template,
    LOCAL_CONFIG_TEMPLATE,
)
from daktari.resource_utils import get_resource

config_path = Path("./.daktari.config")
current_version: Version = Version(__version__)
next_version = version.parse(f"{current_version.major}.{current_version.minor}.{current_version.micro + 1}")

TEST_CHECKS = [
    EnvVarSet(variable_name="SOME_ENV_VAR"),
    IntelliJIdeaInstalled(),
    IntelliJProjectImported(),
]


class TestConfig(unittest.TestCase):
    def tearDown(self):
        if os.path.exists(LOCAL_CONFIG_PATH):
            os.remove(LOCAL_CONFIG_PATH)

    def test_empty_version(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            result = check_version_compatibility(config_path, "")
            self.assertEqual(result, True)
            self.verify_warning(
                fake_out,
                "⚠️  No minimum version found in .daktari.config. Specifying daktari_version is "
                "recommended to ensure team members have compatible versions installed.",
            )

    def test_invalid_version(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            result = check_version_compatibility(config_path, 'daktari_version="....9"')
            self.assertEqual(result, False)
            self.verify_error(fake_out, "❌  Invalid daktari_version in .daktari.config: ....9")

    def test_correct_version(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            result = check_version_compatibility(config_path, f'daktari_version="{current_version}"')
            self.assertEqual(result, True)
            self.verify_no_logging(fake_out)

    def test_too_old_version(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            result = check_version_compatibility(config_path, f'daktari_version="{next_version}"')
            self.assertEqual(result, False)
            self.verify_error(
                fake_out,
                f"❌  Installed version of daktari [{current_version}] is "
                f"too old for this project (needs at least {next_version}). ",
            )

    def test_invalid_version_returns_none(self):
        result = parse_raw_config(config_path, f'daktari_version="{next_version}"')
        self.assertEqual(result, None)

    def test_invalid_config_prints_error(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            result = parse_raw_config(config_path, "checks = [NonExistentCheck()]")
            self.assertEqual(result, None)
            self.verify_error(fake_out, "❌  Failed to parse .daktari.config - config is not valid.")

    def test_local_config_does_not_exist(self):
        config = Config(None, None, TEST_CHECKS)
        updated_config = apply_local_config(config)
        self.assertEqual(config, updated_config)

    def test_local_config_invalid_yml(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            self.write_to_local_config("{{ invalid yaml }")

            config = Config(None, None, TEST_CHECKS)
            updated_config = apply_local_config(config)
            self.assertEqual(None, updated_config)
            self.verify_error(
                fake_out, f"❌  Failed to parse {LOCAL_CONFIG_PATH} - config is not valid YAML. Error follows."
            )

    def test_local_config_empty_file(self):
        self.write_to_local_config("")

        config = Config(None, None, TEST_CHECKS)
        updated_config = apply_local_config(config)
        self.assertEqual(config, updated_config)

    def test_local_config_empty_ignored_checks(self):
        self.write_to_local_config("ignoredChecks: []")

        config = Config(None, None, TEST_CHECKS)
        updated_config = apply_local_config(config)
        self.assertEqual(config, updated_config)

    def test_local_config_always_quiet(self):
        self.write_to_local_config("alwaysQuiet: true")

        config = Config(None, None, TEST_CHECKS)
        expected_config = replace(config, quiet_mode=True)
        updated_config = apply_local_config(config)
        self.assertEqual(expected_config, updated_config)

    def test_generate_local_config(self):
        write_local_config_template()

        expected_contents = get_resource(LOCAL_CONFIG_TEMPLATE)
        with open(LOCAL_CONFIG_PATH, "r", encoding="utf-8") as local_config_file:
            actual_contents = local_config_file.read()
            self.assertEqual(expected_contents, actual_contents)

    def test_generate_local_config_overwrites(self):
        write_local_config_template()
        write_local_config_template()

        expected_contents = get_resource(LOCAL_CONFIG_TEMPLATE)
        with open(LOCAL_CONFIG_PATH, "r", encoding="utf-8") as local_config_file:
            actual_contents = local_config_file.read()
            self.assertEqual(expected_contents, actual_contents)

    def test_version_number_sanitised(self):
        result = sanitise_version_string("9.22")
        self.assertEqual("9.22.0", result)

    def test_local_config_template(self):
        template_text = get_resource(LOCAL_CONFIG_TEMPLATE)
        self.write_to_local_config(template_text)

        config = Config(None, None, TEST_CHECKS)
        updated_config = apply_local_config(config)
        self.assertEqual(config, updated_config)

    def test_local_config_ignored_check_dynamic_name(self):
        self.write_to_local_config("ignoredChecks: ['env.variableSet.SOME_ENV_VAR']")

        config = Config(None, None, TEST_CHECKS)
        updated_config = apply_local_config(config)
        self.assertEqual([IntelliJIdeaInstalled(), IntelliJProjectImported()], updated_config.checks)
        self.assertEqual([EnvVarSet(variable_name="SOME_ENV_VAR")], updated_config.ignored_checks)

    def test_local_config_ignored_check_with_dependants(self):
        self.write_to_local_config("ignoredChecks: ['intellij.installed']")

        config = Config(None, None, TEST_CHECKS)
        updated_config = apply_local_config(config)
        self.assertEqual([EnvVarSet(variable_name="SOME_ENV_VAR")], updated_config.checks)
        self.assertEqual([IntelliJIdeaInstalled(), IntelliJProjectImported()], updated_config.ignored_checks)

    def write_to_local_config(self, contents: str):
        with open(LOCAL_CONFIG_PATH, "a") as local_config_file:
            local_config_file.write(contents)

    def verify_no_logging(self, fake_out: StringIO):
        self.assertEqual(fake_out.getvalue(), "")

    def verify_warning(self, fake_out: StringIO, warning_text: str):
        self.assertIn(yellow(warning_text), fake_out.getvalue())

    def verify_error(self, fake_out: StringIO, warning_text: str):
        self.assertIn(red(warning_text), fake_out.getvalue())


if __name__ == "__main__":
    unittest.main()



import unittest
from unittest import mock

from daktari.version_utils import get_simple_cli_version


class TestVersionUtils(unittest.TestCase):
    @mock.patch("daktari.version_utils.get_stdout", return_value="1.2.3")
    def test_get_simple_cli_version(self, _):
        self.assertEqual(get_simple_cli_version("foo"), "1.2.3")

    @mock.patch("daktari.version_utils.get_stdout", return_value="pre-commit 4.5.1")
    def test_get_simple_cli_version_with_binary_name(self, _):
        self.assertEqual(get_simple_cli_version("foo"), "4.5.1")

    @mock.patch("daktari.version_utils.get_stdout", return_value="pre-commit has an update available")
    def test_get_simple_cli_version_no_match(self, _):
        self.assertEqual(get_simple_cli_version("foo"), None)

    @mock.patch("daktari.version_utils.get_stdout", return_value="pre-commit 100")
    def test_get_simple_cli_version_invalid_version(self, _):
        self.assertEqual(get_simple_cli_version("foo"), None)



import unittest

from daktari.check_utils import get_all_dependent_check_names, CyclicCheckException
from daktari.test_check_factory import DummyCheck


class TestCheckUtils(unittest.TestCase):
    def test_get_all_dependent_check_names(self):
        # Dummy checks set up with dependencies as follows (A <- B means "B depends on A")
        #
        #   A <- B <- C <- E
        #             D <- E
        #
        check_a = DummyCheck("A")
        check_b = DummyCheck("B", [check_a])
        check_c = DummyCheck("C", [check_b])
        check_d = DummyCheck("D")
        check_e = DummyCheck("E", [check_c, check_d])

        self.assertEqual(set(), get_all_dependent_check_names(check_a))
        self.assertEqual({"A"}, get_all_dependent_check_names(check_b))
        self.assertEqual({"A", "B"}, get_all_dependent_check_names(check_c))
        self.assertEqual({"A", "B", "C", "D"}, get_all_dependent_check_names(check_e))

    def test_self_cycle(self):
        check = DummyCheck("A")
        check.depends_on = [check]
        with self.assertRaises(CyclicCheckException):
            get_all_dependent_check_names(check)

    def test_simple_cycle(self):
        check_a = DummyCheck("A")
        check_b = DummyCheck("B")

        check_a.depends_on = [check_b]
        check_b.depends_on = [check_a]
        with self.assertRaises(CyclicCheckException):
            get_all_dependent_check_names(check_a)

    def test_larger_cycle(self):
        check_a = DummyCheck("A")
        check_b = DummyCheck("B")
        check_c = DummyCheck("C")

        check_a.depends_on = [check_b]
        check_b.depends_on = [check_c]
        check_c.depends_on = [check_a]

        with self.assertRaises(CyclicCheckException):
            get_all_dependent_check_names(check_a)

    def test_depends_on_cycle(self):
        check_a = DummyCheck("A")
        check_b = DummyCheck("B")
        check_c = DummyCheck("C")
        check_d = DummyCheck("D")

        check_a.depends_on = [check_b]
        check_b.depends_on = [check_c]
        check_c.depends_on = [check_d]
        check_d.depends_on = [check_b]

        with self.assertRaises(CyclicCheckException):
            get_all_dependent_check_names(check_a)


if __name__ == "__main__":
    unittest.main()



import logging
from os import environ

import distro


class OS:
    OS_X = "osx"
    GENERIC = "generic"
    UBUNTU = "ubuntu"


def detect_os() -> str:
    (id_name, _, _) = distro.linux_distribution()
    if id_name == "Darwin":
        return OS.OS_X
    elif id_name == "Ubuntu":
        return OS.UBUNTU
    else:
        return OS.GENERIC


def check_env_var_exists(variable) -> bool:
    if environ.get(variable) is not None:
        return True
    else:
        logging.debug("Variable has returned empty", exc_info=True)
        return False


def get_env_var_value(variable) -> str:
    if environ.get(variable) is not None:
        return str(environ.get(variable))
    else:
        logging.debug("Variable is not set", exc_info=True)
        return ""



import logging
from typing import List, Set

from colors import yellow

from daktari.check import Check, CheckStatus, CheckResult
from daktari.check_sorter import sort_checks
from daktari.os import detect_os
from daktari.result_printer import print_check_result


def run_checks(checks: List[Check], quiet_mode: bool, fail_fast: bool) -> bool:
    return CheckRunner(checks, quiet_mode, fail_fast).run()


class CheckRunner:
    def __init__(self, checks: List[Check], quiet_mode: bool, fail_fast: bool):
        self.checks = [check for check in checks if check.run_on is None or check.run_on == detect_os()]
        self.all_passed = True
        self.checks_passed: Set[str] = set()
        self.quiet_mode = quiet_mode
        self.fail_fast = fail_fast

    def run(self) -> bool:
        for idx, check in enumerate(sort_checks(self.checks)):
            self.try_run_check(idx, check)
            if self.early_exit():
                break

        return self.all_passed

    def early_exit(self) -> bool:
        return self.fail_fast and not self.all_passed

    def try_run_check(self, idx: int, check: Check):
        dependencies_met = all([dependency.name in self.checks_passed for dependency in check.depends_on])
        if dependencies_met:
            self.run_check(idx, check)
        else:
            self.diagnose_missing_dependency(check)

    def run_check(self, idx: int, check: Check):
        logging.info(f"Running check {check.name}")
        result = self.run_check_in_try(check)
        if result.status in (CheckStatus.PASS, CheckStatus.PASS_WITH_WARNING):
            self.checks_passed.add(check.name)
        else:
            self.all_passed = False

        print_check_result(result, self.early_exit(), self.quiet_mode, idx, len(self.checks))

    def run_check_in_try(self, check: Check) -> CheckResult:
        try:
            return check.check()
        except Exception as err:
            logging.debug(f"Exception running check {check.name}", exc_info=True)
            return CheckResult(check.name, CheckStatus.ERROR, f"Check failed with unhandled {type(err).__name__}", {})

    def diagnose_missing_dependency(self, check: Check):
        all_checks = {check.name for check in self.checks}
        check_dependencies = {dependency.name for dependency in check.depends_on}
        missing_checks = check_dependencies.difference(all_checks)
        if check_dependencies.difference(all_checks):
            print(f"⚠️  [{yellow(check.name)}] skipped due to missing dependent checks: {', '.join(missing_checks)}")
        else:
            print(f"⚠️  [{yellow(check.name)}] skipped due to previous failures")



import unittest
from io import StringIO
from unittest.mock import patch

from colors import red, yellow, green

from daktari.check_runner import run_checks
from daktari.test_check_factory import DummyCheck, ExplodingCheck


class TestCheckRunner(unittest.TestCase):
    def test_status_all_passing(self):
        checks = [DummyCheck("check.one", succeed=True), DummyCheck("check.two", succeed=True)]
        result = run_checks(checks, quiet_mode=True, fail_fast=False)
        self.assertTrue(result)

    def test_runs_all_and_returns_failure_if_check_fails(self):
        final_check = DummyCheck("check.three", succeed=True)
        checks = [DummyCheck("check.one", succeed=True), DummyCheck("check.two", succeed=False), final_check]
        result = run_checks(checks, quiet_mode=True, fail_fast=False)
        self.assertFalse(result)
        self.assertTrue(final_check.was_run)

    def test_aborts_and_returns_failure_if_check_fails_and_fail_fast(self):
        final_check = DummyCheck("check.three", succeed=True)
        checks = [DummyCheck("check.one", succeed=True), DummyCheck("check.two", succeed=False), final_check]
        result = run_checks(checks, quiet_mode=True, fail_fast=True)
        self.assertFalse(result)
        self.assertFalse(final_check.was_run)

    def test_copes_with_unhandled_errors_and_runs_later_checks(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            final_check = DummyCheck("check.three", succeed=True)
            checks = [ExplodingCheck(), final_check]
            result = run_checks(checks, quiet_mode=True, fail_fast=False)
            self.assertFalse(result)
            self.assertTrue(final_check.was_run)
            self.assertIn(f"💥 [{red('exploding.check')}] Check failed with unhandled Exception", fake_out.getvalue())

    def test_skips_dependents_on_failure(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            final_check = DummyCheck("dependent.check", depends_on=[ExplodingCheck], succeed=True)
            checks = [ExplodingCheck(), final_check]
            result = run_checks(checks, quiet_mode=True, fail_fast=False)
            self.assertFalse(result)
            self.assertFalse(final_check.was_run)
            self.assertIn(f"⚠️  [{yellow('dependent.check')}] skipped due to previous failures", fake_out.getvalue())

    def test_skips_missing_dependencies(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            missing_check = DummyCheck("check.one")
            final_check = DummyCheck("dependent.check", depends_on=[missing_check])
            checks = [final_check]
            result = run_checks(checks, quiet_mode=True, fail_fast=False)
            self.assertTrue(result)
            self.assertFalse(final_check.was_run)
            self.assertIn(
                f"⚠️  [{yellow('dependent.check')}] skipped due to missing dependent checks: check.one",
                fake_out.getvalue(),
            )

    def test_outputs_success(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            checks = [DummyCheck("check.one", succeed=True)]
            result = run_checks(checks, quiet_mode=False, fail_fast=False)
            self.assertTrue(result)
            self.assertIn(f"✅ [{green('check.one')}] dummy check", fake_out.getvalue())

    def test_suppresses_success_in_quiet_mode(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            checks = [DummyCheck("check.one", succeed=True)]
            result = run_checks(checks, quiet_mode=True, fail_fast=False)
            self.assertTrue(result)
            self.assertNotIn("✅", fake_out.getvalue())

    def test_outputs_failure_in_quiet_mode(self):
        with patch("sys.stdout", new=StringIO()) as fake_out:
            checks = [DummyCheck("check.one", succeed=False)]
            result = run_checks(checks, quiet_mode=True, fail_fast=False)
            self.assertFalse(result)
            self.assertIn(f"❌ [{red('check.one')}] dummy check", fake_out.getvalue())



import unittest
from unittest import mock

from daktari.check import CheckStatus
from daktari.checks.certs import CertificateIsNotExpired
from daktari.resource_utils import get_resource_path


class TestCertificateIsNotExpired(unittest.TestCase):
    @mock.patch("daktari.checks.certs.crypto.load_certificate")
    def test_passes_with_warning_if_cannot_determine_expiry(self, mock_get_not_after):
        mock_get_not_after.return_value.get_notAfter.return_value = None
        check = CertificateIsNotExpired(get_resource_path("mock_cert.pem").__str__())
        result = check.check()
        self.assertEqual(result.status, CheckStatus.PASS_WITH_WARNING)
        self.assertEqual(result.summary, "Unable to determine expiry date of mock_cert.pem")

    @mock.patch("daktari.checks.certs.crypto.load_certificate")
    def test_passes_for_valid_cert(self, mock_get_not_after):
        mock_get_not_after.return_value.get_notAfter.return_value = b"20501231235959Z"
        check = CertificateIsNotExpired(get_resource_path("mock_cert.pem").__str__())
        result = check.check()
        self.assertEqual(result.status, CheckStatus.PASS)
        self.assertEqual(result.summary, "mock_cert.pem is not expired")

    @mock.patch("daktari.checks.certs.crypto.load_certificate")
    def test_fails_for_expired_cert(self, mock_get_not_after):
        mock_get_not_after.return_value.get_notAfter.return_value = b"20201231235959Z"
        check = CertificateIsNotExpired(get_resource_path("mock_cert.pem").__str__())
        result = check.check()
        self.assertEqual(result.status, CheckStatus.FAIL)
        self.assertEqual(result.summary, "mock_cert.pem expired on 2020-12-31 23:59:59")



from daktari.check import Check, CheckResult


class DummyCheck(Check):
    def __init__(self, name: str = "check.name", depends_on=None, succeed: bool = True):
        if depends_on is None:
            depends_on = []
        self.name = name
        self.depends_on = depends_on
        self.was_run = False
        self.succeed = succeed

    def check(self) -> CheckResult:
        self.was_run = True
        return self.verify(self.succeed, "dummy check")


class ExplodingCheck(Check):
    name = "exploding.check"

    def check(self) -> CheckResult:
        raise Exception("boom")


class DummyCheck2(Check):
    def __init__(self, name: str = "check.name", depends_on=None):
        if depends_on is None:
            depends_on = []
        self.name = name
        self.depends_on = depends_on

    def check(self) -> CheckResult:
        return self.passed("no check")



from pathlib import Path

from daktari.command_utils import get_stdout

import os
import re


def get_absolute_path(path: str) -> str:
    return os.path.expanduser(path)


def is_ascii(path: str) -> bool:
    file_output = get_stdout(["file", path]) or ""
    parts = file_output.strip().split(": ")
    return parts[1] == "ASCII text"


def file_exists(path: str) -> bool:
    testing_file = Path(path)
    return testing_file.is_file()


def file_contains_text(path: str, text: str) -> bool:
    if not file_exists(path):
        return False
    with open(path, "r") as file:
        for line in file:
            if text in line:
                return True
    return False


def file_contains_text_regex(path: str, regex: str) -> bool:
    if not file_exists(path):
        return False
    with open(path, "r") as file:
        for line in file:
            if re.search(regex, line):
                return True
    return False


def dir_exists(path: str) -> bool:
    testing_dir = Path(path)
    return testing_dir.is_dir()
